package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.dao.PerformanceBenchmarkInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.service.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

/**
 * @author hlf
 * @date 2024/3/28 14:07
 * 文件说明：
 */
@Service
public class PerformanceBenchmarkInfoServiceImpl extends ServiceImpl<PerformanceBenchmarkInfoMapper, PerformanceBenchmarkInfo> implements IPerformanceBenchmarkInfoService {

	@Resource
	private IBenchmarkingRecordService benchmarkingRecordService;

	@Resource
	private IEvaluateRuleService evaluateRuleService;

	@Resource
	private IEvaluateRuleInfoService evaluateRuleInfoService;

	@Resource
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Resource
	private IOrganizationEvaluationInfoService organizationEvaluationInfoService;

	@Override
	public IPage<PerformanceBenchmarkInfo> listPage(Integer pageNum, Integer pageSize, String toolCategory, String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		int isBg = 0;
		int isDb = 0;
		int isCb = 0;
		if ("寻标".equals(toolCategory)) {
			xb(manageCategory, checkCycle, degreeYear, quarterlyMonth);
		} else if ("立标".equals(toolCategory)) {
			lb(manageCategory, checkCycle, degreeYear, quarterlyMonth);
		} else if ("标杆".equals(toolCategory)) {
			bg(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			isBg = 1;
		} else if ("达标".equals(toolCategory)) {
			db(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			isDb = 2;
		} else if ("超标".equals(toolCategory)) {
			cb(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			isCb = 3;
		}
		//构造分页构造器
		Page<PerformanceBenchmarkInfo> page = new Page<>(pageNum, pageSize);
		return baseMapper.selectListPage(page, manageCategory, checkCycle, degreeYear, quarterlyMonth, isBg, isDb, isCb);
	}

	/**
	 * 寻标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void xb(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "寻标");
		if (benchmarkingRecordList.size() == 0) {
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = new ArrayList<>();
			EvaluateRule evaluateRule = evaluateRuleService.getEvaluateRuleListByYear(String.valueOf(degreeYear), checkCycle, manageCategory);
			if (null != evaluateRule) {
				List<EvaluateRuleInfo> evaluateRuleInfoList = evaluateRuleInfoService.getEvaluateRuleInfoByEvaluateRuleId(evaluateRule.getId());
				for (EvaluateRuleInfo evaluateRuleInfo : evaluateRuleInfoList) {
					List<IndicatorDictionary> indicatorDictionaryList = indicatorDictionaryService.getIndicatorDictionaryList(evaluateRuleInfo.getIndicatorId());
					if (indicatorDictionaryList.size() > 0) {
						//计划值
						IndicatorDictionary jhz = indicatorDictionaryList.stream().filter(p -> !p.getOptionCode().equals("WCZ")).findFirst().orElse(null);
						//实际完成值
						IndicatorDictionary wcz = indicatorDictionaryList.stream().filter(p -> p.getOptionCode().equals("WCZ")).findFirst().orElse(null);
						if (jhz != null && wcz != null) {
							List<OrganizationEvaluationInfo> jhzList = organizationEvaluationInfoService.getOrganizationEvaluationInfoListByIndicatorDictionaryId(jhz.getId(), String.valueOf(degreeYear), String.valueOf(quarterlyMonth));
							List<OrganizationEvaluationInfo> wczList = organizationEvaluationInfoService.getOrganizationEvaluationInfoListByIndicatorDictionaryId(wcz.getId(), String.valueOf(degreeYear), String.valueOf(quarterlyMonth));
							if (jhzList.size() == wczList.size()) {
								for (int i = 0; i < wczList.size(); i++) {
									OrganizationEvaluationInfo jhzObj = jhzList.get(i);
									OrganizationEvaluationInfo wczObj = wczList.get(i);
									PerformanceBenchmarkInfo performanceBenchmarkInfo = new PerformanceBenchmarkInfo();
									performanceBenchmarkInfo.setId(Convert.toStr(IdUtil.getSnowflake(1, 1).nextId()));
									performanceBenchmarkInfo.setCheckCycle(wczObj.getCheckCycle());
									performanceBenchmarkInfo.setDegreeYear(degreeYear);
									if (!"NDKP".equals(checkCycle)) {
										performanceBenchmarkInfo.setQuarterlyMonth(quarterlyMonth);
									}
									performanceBenchmarkInfo.setManageCategory(manageCategory);
									performanceBenchmarkInfo.setCompanyId(wczObj.getOrganizationId());
									performanceBenchmarkInfo.setCompanyName(wczObj.getCompanyName());
									performanceBenchmarkInfo.setCompanyShort(wczObj.getCompanyShort());
									performanceBenchmarkInfo.setInstallCapacity(wczObj.getInstallCapacity());
									performanceBenchmarkInfo.setIndexName(evaluateRuleInfo.getIndicatorName());
									performanceBenchmarkInfo.setPlannedValue(jhzObj.getQuantifiedValue());
									performanceBenchmarkInfo.setCompleteValue(wczObj.getQuantifiedValue());
									performanceBenchmarkInfo.setIsBg(0);
									performanceBenchmarkInfo.setIsDb(0);
									performanceBenchmarkInfo.setIsCb(0);
									performanceBenchmarkInfoList.add(performanceBenchmarkInfo);
								}
							}
						}
					}
				}
			}
			super.saveBatch(performanceBenchmarkInfoList);
			addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "寻标");
		}
	}

	/**
	 * 立标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void lb(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "立标");
		if (benchmarkingRecordList.size() == 0) {
			if ("NDKP".equals(checkCycle)) {
				List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, null);
				double completeValueSum = performanceBenchmarkInfoList.stream().mapToDouble(PerformanceBenchmarkInfo::getCompleteValue).sum();
				double installCapacitySum = performanceBenchmarkInfoList.stream().mapToDouble(PerformanceBenchmarkInfo::getInstallCapacity).sum();
				List<PerformanceBenchmarkInfo> updateList = new ArrayList<>();
				performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {
					PerformanceBenchmarkInfo.setRecommendedValue(yearIndexRecommendedValueA(completeValueSum, installCapacitySum, PerformanceBenchmarkInfo.getInstallCapacity()));
					updateList.add(PerformanceBenchmarkInfo);
				});
				super.updateBatchById(updateList);
			} else if ("JDKP".equals(checkCycle) || "YDKP".equals(checkCycle)) {
				List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
				List<PerformanceBenchmarkInfo> updateList = new ArrayList<>();


			}
			addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "立标");
		}
	}

	/**
	 * 标杆
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void bg(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "标杆");
		if (benchmarkingRecordList.size() == 0) {
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {
				PerformanceBenchmarkInfo.setBgVal(PerformanceBenchmarkInfo.getInstallCapacity() / PerformanceBenchmarkInfo.getCompleteValue());
			});
			performanceBenchmarkInfoList.sort(Comparator.comparingDouble(PerformanceBenchmarkInfo::getBgVal).reversed());
			if (!performanceBenchmarkInfoList.isEmpty() && performanceBenchmarkInfoList.get(0).getBgVal() != 0) {
				performanceBenchmarkInfoList.get(0).setIsBg(1);
			}
			super.updateBatchById(performanceBenchmarkInfoList);
			addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "标杆");
		}
	}

	/**
	 * 达标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void db(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "达标");
		if (benchmarkingRecordList.size() == 0) {
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			for (PerformanceBenchmarkInfo performanceBenchmarkInfo : performanceBenchmarkInfoList) {
				if (performanceBenchmarkInfo.getCompleteValue() >= performanceBenchmarkInfo.getScalingValue()) {
					performanceBenchmarkInfo.setIsDb(1);
					performanceBenchmarkInfo.setDbVal(performanceBenchmarkInfo.getCompleteValue() - performanceBenchmarkInfo.getScalingValue());
				}
			}
			super.updateBatchById(performanceBenchmarkInfoList);
			addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "达标");
		}
	}

	/**
	 * 超标
	 *
	 * @param manageCategory 经营类别（火电、水电、新能源(风电、光伏)、煤电、海外、综合）
	 * @param checkCycle     周期（NDKP、JDKP、YDKP）
	 * @param degreeYear     年度
	 * @param quarterlyMonth 季/月度
	 */
	private void cb(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth) {
		List<BenchmarkingRecord> benchmarkingRecordList = benchmarkingRecordService.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth, "超标");
		if (benchmarkingRecordList.size() == 0) {
			List<PerformanceBenchmarkInfo> performanceBenchmarkInfoList = baseMapper.selectList(manageCategory, checkCycle, degreeYear, quarterlyMonth);
			performanceBenchmarkInfoList.stream().filter(p -> p.getIsBg().equals(1)).findFirst().ifPresent(performanceBenchmarkInfo -> performanceBenchmarkInfoList.forEach(PerformanceBenchmarkInfo -> {
				if (PerformanceBenchmarkInfo.getCompleteValue() > performanceBenchmarkInfo.getScalingValue()) {
					PerformanceBenchmarkInfo.setIsCb(1);
					PerformanceBenchmarkInfo.setCbVal(PerformanceBenchmarkInfo.getCompleteValue() - performanceBenchmarkInfo.getScalingValue());
				}
			}));
			super.updateBatchById(performanceBenchmarkInfoList);
			addRecord(manageCategory, checkCycle, degreeYear, quarterlyMonth, "超标");
		}
	}

	/**
	 * 年度指标推荐值A
	 */
	private Double yearIndexRecommendedValueA(double a, double b, double c) {
		return (a / b) * c;
	}

	/**
	 * 年度指标推荐值B
	 */
	private Double yearIndexRecommendedValueB(double a, double b, double c) {
		double B;
		if (b == 0) {
			B = a * 0.02;
		} else {
			B = (a / b) * c;
		}
		return B;
	}

	/**
	 * 季/月度指标推荐值
	 */
	private Double quarterlyMonthIndexRecommendedValue(double a, double b) {
		return a * b;
	}

	private void addRecord(String manageCategory, String checkCycle, Integer degreeYear, Integer quarterlyMonth, String toolCategory) {
		BenchmarkingRecord benchmarkingRecord = new BenchmarkingRecord();
		benchmarkingRecord.setId(Convert.toStr(IdUtil.getSnowflake(1, 1).nextId()));
		benchmarkingRecord.setManageCategory(manageCategory);
		benchmarkingRecord.setCheckCycle(checkCycle);
		benchmarkingRecord.setDegreeYear(degreeYear);
		benchmarkingRecord.setQuarterlyMonth(quarterlyMonth);
		benchmarkingRecord.setTagType(toolCategory);
		benchmarkingRecordService.save(benchmarkingRecord);
	}
}
